/* -*- Mode: java; tab-width: 8; c-basic-offset: 4 -*-
 * Copyright  1997, 1998 Netscape Communications Corporation, All Rights Reserved.
 */

package com.netscape.javascript.CORBA;

import com.netscape.javascript.*;
import org.omg.CORBA.TCKind;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.Any;
import org.omg.CORBA.portable.*;
import org.omg.CORBA.CompletionStatus;

/**
 * This class reflects a IDL arrays and sequences into the JS
 * environment.
 *
 * @author Ruslan Belkin
 */

public class   Array    extends Simple  {

    public Array (Any any)
    {
    	super (any);
    }

    public Array (TypeCode tc)
    {
    	super (tc);
    }

    public String getClassName ()
    {
        return any.type ().kind ().value () == TCKind._tk_sequence ? "Sequence" : "Array";
    }

    public String toString ()
    {
        String s = getClassName () + ":";

        try {
            s += any.type ().id ();
        }
        catch (Exception e) {   s+= "unknown";  };

        s += "={";
        for (int i = 0; i < members.length; i++)
        {
            s += members[i];
            if (i != members.length - 1)    s += ",";
        }
        s += "}";
        return s;
    }

    public Object valueOf ()
    {
        return this;
    }

    public boolean has (int index, Scriptable start)
    {
        try {
            return index >= 0 && index < any.type ().length ();
        }
        catch (org.omg.CORBA.UserException ue)  { /* normally should never happen */ }
        return false;
    }

    public void put (int index, Scriptable start, Object value)
    {
        if (!has (index, start))
            return;

        Object current = members[index];
        members[index] = value;
        try {
            encode ();
        }
        catch (org.omg.CORBA.SystemException se)
        {
            members[index] = current;
            Init.raise (new SystemException (se));
        }
    }

    public Object  get (int index, Scriptable start)
    {
        return has (index, start) ? members[index] : Scriptable.NOT_FOUND;
    }

    public  static Object extractObject (InputStream is, TypeCode tc)
        throws org.omg.CORBA.MARSHAL
    {
        switch (tc.kind().value ())
        {
            case TCKind._tk_array    :
            case TCKind._tk_sequence :

                Any any = Init.orb.create_any ();
                any.type (tc);

                OutputStream os = any.create_output_stream ();
                copyObject (is, os, tc);
                any.read_value (os.create_input_stream (), tc);
                return new Array (any);
            default :
                return Marshal.extractObject (is, tc);
        }
    }

    public static void copyObject (InputStream is, OutputStream os, TypeCode tc)
        throws org.omg.CORBA.MARSHAL
    {
        int tkind = tc.kind().value();

        try {
            switch (tkind)
            {
                case TCKind._tk_array    :
                case TCKind._tk_sequence :
                    if (tkind == TCKind._tk_sequence)
                        os.write_long (is != null ? is.read_long () : tc.length ());

                    for (int i = 0; i < tc.length (); i++)
                        copyObject (is, os, tc.content_type ());
                    break;
                default:
                    Marshal.copyObject (is, os, tc);
            }
        }
        catch (org.omg.CORBA.UserException e)  { /* should never happen */ }
    }

    public static void writeObject (OutputStream os, TypeCode tc, Object value)
        throws org.omg.CORBA.MARSHAL
    {
        int tkind  = tc.kind().value ();
        boolean ok = true;

        try {
            switch (tkind)
            {
                case TCKind._tk_array    :
                case TCKind._tk_sequence :
                    if (ok = value instanceof Array)
                    {
                        if (tkind == TCKind._tk_sequence)
                            os.write_long (tc.length ());

                        for (int i = 0; i < tc.length (); i++)
                            writeObject (os, tc.content_type (), ((Array)value).get (i, null));
                    }
                    break;
                default: Marshal.writeObject (os, tc, value);
            }
        }
        catch (org.omg.CORBA.UserException e)   { /* should never happend */ }

        if (!ok)
            throw new org.omg.CORBA.MARSHAL ("cannot cast value: " + value + " to type: " + tc, 0, CompletionStatus.COMPLETED_NO);
    }

    public void decode ()
        throws org.omg.CORBA.MARSHAL
    {
        TypeCode tc = any.type ();
        int tkind = tc.kind().value();

        try {
            InputStream is = any.create_input_stream ();

            members = new Object [tc.length ()];

            for (int i = 0; i < members.length; i++)
                members[i] = extractObject (is, tc.content_type ());
        }
        catch (org.omg.CORBA.UserException e)   { /* normally should never happend */  }
    }

    public static boolean handledType (int tkind)
    {
        switch (tkind)
        {
            case TCKind._tk_array    :
            case TCKind._tk_sequence :
                return true;
        }
        return false;
    }

    Object [] members;
